include <sizes.scad>
include <MCAD/involute_gears.scad>

/* various things to 'draw' */
//translate([0,0,6]) gears(0);
//translate([0,0,3.4]) gears(1);
//gears(2);
//gears(3);

/* facts about the gears and motor torque */
teeth = [9,22,13,25];
radius = [6,6*teeth[1]/teeth[0],7,7*teeth[3]/teeth[2]];
pulley_radius = 12.52/2; //mm
motor_torque = 6000; //g.mm
motor_steps_per_rev = 200 * 16; // i.e. 16x microstepping
ratio = teeth[1]/teeth[0]*teeth[3]/teeth[2]*2;
echo("ratio is", ratio);
echo("force is", ratio*motor_torque/pulley_radius, "g");
echo("steps per mm is", ratio*motor_steps_per_rev/(2*pi*pulley_radius));

module gears(which) {
	circular_pitch=radius[which]/teeth[which]*360;
	thickness=5;

	axis_angle=90;
	outside_pitch_radius = teeth * circular_pitch / 360;
	pitch_apex0=outside_pitch_radius[1] * sin (axis_angle) + 
		(outside_pitch_radius[1] * cos (axis_angle) + outside_pitch_radius[0]) / tan (axis_angle);
	cone_distance = sqrt (pow (pitch_apex0, 2) + pow (outside_pitch_radius[0], 2));
 
	if (which == 0) {
		difference() {
			union() {
				bevel_gear (outside_circular_pitch=circular_pitch, 
					cone_distance=cone_distance,
					face_width=thickness,
					bore_diameter=0,
					number_of_teeth=teeth[which]);
				translate([0,0,-6])
					cylinder(r=8,h=6);
			}
			translate([0,0,-7])
				cylinder(r=m5_clearance/2,h=100);
			translate([0,0,-3]) {
				rotate([0,90,0])	cylinder(r=m3_clearance/2, h=100);
				//rotate([0,90,0])	cylinder(r=m3_nut/sqrt(3),h=5.5,$fn=6);
				//translate([0,-m3_nut/2,-10])	cube(size=[5.5,m3_nut,10]);
			}
		}
 	}

	if (which == 1) {
		difference() {
			union() {
				bevel_gear (outside_circular_pitch=circular_pitch,  
					cone_distance=cone_distance,
					face_width=thickness,
					finish=1,
					gear_thickness=2,
					bore_diameter=m4_clearance,
					number_of_teeth=teeth[which]);
				translate([0,0,-1.5])
					cylinder(r=10,h=2);
			}
			translate([0,0,-2.25])
				cylinder(r=m4_nut/sqrt(3),h=3,$fn=6);
		}
 	}

	if (which == 2) {
		difference() {
			gear (circular_pitch=circular_pitch,  
				gear_thickness = thickness,  
				rim_thickness = thickness,  
				hub_thickness = thickness,  
				bore_diameter=m4_clearance,
				number_of_teeth=teeth[which]);
			translate([0,0,thickness-3])
				cylinder(r=m4_nut/sqrt(3),h=4,$fn=6);
		}
 	}

	if (which == 3) {
		difference() {
			union() {
				gear (circular_pitch=circular_pitch,  
					gear_thickness = thickness,  
					rim_thickness = thickness,  
					hub_thickness = thickness,
					bore_diameter=0,
					number_of_teeth=teeth[which]);
				translate([0,0,thickness+1])
					import("XL-8t-8.stl");
			}
			cylinder(r=m4_clearance/2, h=2*thickness+50, center=true);
		}
 	}
}
