include <SY.scad>
include <sizes.scad>
include <gears.scad>

/* various things to 'draw' */
assembly(gears=true, props=true);
//assembly(gears01=true);
//assembly(gears23=true);

//translate([0,0,6]) gears(0);
//translate([0,0,3.4]) gears(1);
//gears(2);
//gears(3);
//motor_mount();
//motor_mount(flag_length=15);
//syringe_mount();
//bearing_mount();
//clamp();
//syringe_idler();
//syringe_idler(cross=true, belt_width=8);



/* locations of various items */
$fn=36;
X=0;Y=1;Z=2;

gear1 = [-8,  10, 14.5];
gear0 = gear1 + [radius[1], radius[0], 0];
motor = [18, gear0[Y], gear0[Z]];
gear2 = [gear1[X], -37, gear1[Z]];
gear3 = gear2+[sqrt(pow(radius[2]+radius[3],2)-pow(15,2)), 0, 15];
idler = gear3-[pulley_radius+1+m4_bearing_od/2,-13.5,0];
syringe = [27,-26, -5];
clamp = syringe + [17,0,5];

mountings = [[7.5,15], [32.5,15], [32.5,38], [7.5,38]];
syringe_z=0;
clamp_spacing = 15;
syringe_clearance = 13.6 + 2;



module assembly(gears=true, gears01=false, gears23=false, props=false) {
	/* props */
	if (props) {
		color("white")	translate(syringe)	syringe();
		color("grey")	translate([0,0,10])	rotate([90,0,0])	import("x-carriage.stl");
	}

	translate([0,0,7])	{
		/* mountings */
		color("orange")	rotate([-90,0,0])	motor_mount();
		color("orange")	translate([0,-15,0]) rotate([90,0,0])	syringe_mount();
		translate(clamp)	rotate([-90,0,90]) clamp();

		translate([0,-45,0]) rotate([90,0,180])bearing_mount();

		/* gears */
		if (gears && !gears23) {
			translate(gear0)	rotate([0,-90,0])	gears(0);
			translate(gear1)	rotate([-90,0,0])	gears(1);
		}
		if (gears && !gears01) {
			translate(gear2)	rotate([-90,0,0])	gears(2);
			translate(gear3)	rotate([-90,0,0])	gears(3);
		}

		/* motor */
		if (props)
			color("white")	translate(motor) rotate([0,90,0]) motor();
	}
}

module motor_mount(flag_length=15) {
	difference() {
		union() {
			linear_extrude(height=1.5) hull() projection(cut = true) motor_mount_solids();
			motor_mount_solids();

			// x-endstop trigger
			translate([mountings[1][X], -mountings[1][Y]])
				hull() {
					cylinder(r=4,h=1.5);
					translate([flag_length-2,-2])
						cube(size=[4,4,1.5]);
				}
		}
		motor_mount_cutouts();
	}
}

module motor_mount_solids() {
	// bearings
	translate([gear1[X],-gear1[Z]])
		bearing_solid();

	// motor mount
	assign(motor_w=28, mount_w=23)
		translate([motor[X],-motor_w/2,0])
		rotate([0,-90,0])
		rotate([0,0,-90])
		linear_extrude(height=2.5) {
			assign(x0=motor_w/2+3, x1=motor_w/2+1, x2= mount_w/2-3,
				y1=0, y2=motor_w/2-mount_w/2+2, y3=motor_w/2+mount_w/2+2) {
					polygon([[x0,y1],[x1,y3],[x2,y3],[x2,y1]]);
					polygon([[-x0,y1],[-x1,y3],[-x2,y3],[-x2,y1]]);
					translate([(x1+x2)/2,y3])	circle(r=(x1-x2)/2);
					translate([-(x1+x2)/2,y3])	circle(r=(x1-x2)/2);
				}
		}

	// mounting holes to carriage
	for (xy = mountings)
		translate([xy[X],-xy[Y]])
			cylinder(r=m3_wide_clearance/2,h=1);
}

module motor_mount_cutouts() {
	// bearings
	translate([gear1[X],-gear1[Z]])
		bearing_cutout();

	// motor mount
	assign(motor_w=28, mount_w=23)
		translate([motor[X],-motor_w/2,0])
		rotate([0,-90,0])
		rotate([0,0,-90])
		translate([0,0,-1])
		linear_extrude(height=10) {
			assign(y1=0, y2=motor_w/2-mount_w/2+2, y3=motor_w/2+mount_w/2+2) {
					translate([mount_w/2,y2])	circle(r=m2_5_clearance/2);
					translate([-mount_w/2,y2])	circle(r=m2_5_clearance/2);	
					translate([mount_w/2,y3])	circle(r=m2_5_clearance/2);
					translate([-mount_w/2,y3])	circle(r=m2_5_clearance/2);
					translate([0,(y2+y3)/2])		circle(r=13);	
			}
		}

	// mounting holes to carriage
	for (xy = mountings)
		translate([xy[X],-xy[Y]])
			carriage_bolt();
}


module syringe_mount() {
	difference() {
		union() {
			linear_extrude(height=1.5) hull() projection(cut = true) syringe_mount_solids();
			intersection() {
				translate([0,0,50]) cube(size=100,center=true);
				syringe_mount_solids();
			}
		}
		syringe_mount_cutouts();
	}
}

module syringe_mount_solids(h=-syringe[Y]-15) {
	// bearings
	translate([gear2[X],gear2[Z]]) bearing_solid();
	translate([gear3[X],gear3[Z]]) bearing_solid();

	// idler bearing
	translate([idler[X], idler[Z], 0])
		assign(h=-idler[Y]-15-1)
			hull() {
				intersection() {
					cylinder(r=m4_bearing_od/2+2,h=h);
					translate([-100,-50,-50]) cube(size=[100,100,100]);
				}
				cylinder(r=6, h=h);
			}

	for (y = [syringe_z, 45])
		translate([syringe[X], y-5, h]) {
			// syringe mounts
			translate([-syringe_clearance/2-4,0,-h])
				cube(size=[syringe_clearance+8, 10, h]);
			rotate([-90,0,0])	cylinder(r=syringe_clearance/2+4,h=10);

			// bearing block mounts
			translate([-2-5, 0, -syringe[Y]-15+syringe_clearance/2-h])
				cube(size=[10,10,2.5+m3_nut_h]);
		}

	// belt clamp
	translate([syringe[X], syringe_z-5, 0])
		cube(size=[syringe_clearance/2+4, 10, h+syringe_clearance/2+4]);

	// mounting holes to carriage
	for (xy = mountings)
		translate(xy) cylinder(r=m3_wide_clearance/2, h=1);
}

module syringe_mount_cutouts() {
	// bearings
	translate([gear2[X],gear2[Z],1]) bearing_cutout();
	translate([gear3[X],gear3[Z],1]) bearing_cutout();

	// idler bearing
	translate([idler[X], idler[Z], -1]) cylinder(r=m4_clearance/2,h=100);
	translate([idler[X], idler[Z], -1]) cylinder(r=m4_nut/sqrt(3),h=6,$fn=6);

	// gear3
	translate([gear3[X],gear3[Z],-gear3[Y]-15-7.5]) cylinder(r=radius[3]+2,h=60);

	// space for syringe
	translate([syringe[X], syringe_z,-syringe[Y]-15])
		rotate([-90,0,0]) rotate_extrude()
			assign(a=syringe_clearance/4,
				b=syringe_clearance/2)
				polygon( points=[[0,-100],[a,-100],[a,-(b-a)/2],[b,(b-a)/2],[b,100],[0,100]]);

	// mounting to bearing_mount
	for (y = [syringe_z, 45])
		translate([syringe[X]-2, y,-syringe[Y]-15]) {
			cylinder(r=m3_clearance/2, h=100); //syringe_clearance/2);
			cylinder(r=m3_nut/sqrt(3), h=syringe_clearance/2+m3_nut_h+1,$fn=6);
		}

	// mouting to belt clamp
	translate([clamp[X], clamp[Z], -clamp[Y]-15])
		for (x = [-1,1])
			translate([0,0,x*clamp_spacing/2]) {
				rotate([0,-90,0]) cylinder(r=m3_clearance/2, h=20);
				translate([-7.5,0,0]) rotate([0,-90,0]) cylinder(r=m3_nut/sqrt(3), h=50,$fn=6);
			}

	// mounting holes to carriage
	for (xy = mountings)
		translate(xy) carriage_bolt();
}


module bearing_mount() {
	mirror([1,0,0])
	difference() {
		union() {
			linear_extrude(height=1.5) hull() projection(cut = true) bearing_mount_solids();
			bearing_mount_solids();
		}
		bearing_mount_cutouts();
	}
}

module bearing_mount_solids(h = 8.5-m3_nut_h) {
	// bearing
	translate([gear3[X],gear3[Z]]) bearing_solid();

	// mounting to syringe_mount
	for (y = [syringe_z, 45])
		translate([syringe[X]-2, y, 0]) {
			cylinder(r=10/2, h=h);
		}
}

module bearing_mount_cutouts() {
	// bearings
	translate([gear3[X],gear3[Z]]) bearing_cutout();

	// mounting to syringe_mount
	for (y = [syringe_z, 45])
		translate([syringe[X]-2, y, -1]) {
			cylinder(r=m3_clearance/2, h=100);
		}
}

module syringe_idler(cross=false, od=22, id=18, belt_width=8) {
	difference() {
		hull() {
			if (cross) {
				cylinder(r=8, h=7.5);
			} else {
				cylinder(r=od/2, h=7.5);
			}

			translate([0,0,10+m4_bearing_od/2])
				rotate([0,90,0]) cylinder(r=m4_bearing_od/2+2, h=belt_width+6, center=true);
		}


		// mounting to syringe
		if (cross) {
			rotate([0,0,45]) {
				cube(size=[100,1.75,10], center=true);
				cube(size=[1.75,100,10], center=true);
			}
		} else {
			cylinder(r=id/2,  h=5*2, center=true);
		}

		translate([0,0,7.5+1+m4_bearing_od/2]) {
			rotate([0,90,0]) cylinder(r=m4_clearance/2, h=100, center=true);
//			rotate([0,90,0]) cylinder(r=m4_bearing_od/2, h=m4_bearing_w, center=true);
			for (x = [-1, 1])
				translate([x*(belt_width/2+3+50),0,0]) rotate([0,90,0]) cylinder(r=m4_nut/sqrt(3),h=100, center=true, $fn=6);
		}
		translate([0,0,50+7.5]) cube(size=[belt_width,100,100], center=true);
		for (y = [-1, 1])
			translate([0,y*(m4_bearing_od/2+50),0]) cube(size=[belt_width,100,100], center=true);
	}
}


module clamp() {
	translate([0,0,2])
		difference() {
			cube(size=[22,10,4], center=true);

			for (x = [-1,1])
				translate([x*clamp_spacing/2,0,0]) cylinder(r=m3_clearance/2, h=20, center=true);
			for (y=5.08*[-1.5,-0.5,0.5,1.5])
				translate([15,y,2]) rotate([-90,0,90]) XL(h=30);
		}		
}




module bearing_solid() {
	cylinder(r=m4_bearing_od/2+2,h=m4_bearing_w+1);
}

module bearing_cutout() {
	translate([0,0,1]) cylinder(r=m4_bearing_od/2,h=m4_bearing_w+1);
	translate([0,0,-5]) cylinder(r=m4_wide_clearance/2,h=m4_bearing_w+10);
}

module carriage_bolt() {
	translate([0,0,-1]) cylinder(r=m3_clearance/2,h=10);  // bolt
	translate([0,0,1.5]) cylinder(r=m3_head_clearance/2,h=100); // head
}

module bearing() {
	cylinder(r=m4_bearing_od/2, h=m4_bearing_w);
}

module motor() {	SY28STH32_A(); }

module syringe() {
	cylinder(r=4.2/2, h=90);
	translate([0,0,9])
		cylinder(r1=4.2/2, r2=13.6/2, h= 3);
	translate([0,0,12])
		cylinder(r=13.6/2, h=64);
	translate([0,0,90-1])
		cylinder(r=13.6/2, h=1);
}

module XL(h)
	{
	linear_extrude(height=h) polygon([[-1.525411,-1],[-1.525411,0],[-1.41777,0.015495],[-1.320712,0.059664],[-1.239661,0.129034],[-1.180042,0.220133],[-0.793044,1.050219],[-0.733574,1.141021],[-0.652507,1.210425],[-0.555366,1.254759],[-0.447675,1.270353],[0.447675,1.270353],[0.555366,1.254759],[0.652507,1.210425],[0.733574,1.141021],[0.793044,1.050219],[1.180042,0.220133],[1.239711,0.129034],[1.320844,0.059664],[1.417919,0.015495],[1.525411,0],[1.525411,-1]]);
	}
